.TH CTAGS 1 "Version 1.6B" "Darren Hiebert"

.SH NAME
ctags \- Generate C language tag files for use with
.BR vi (1)

.SH SYNOPSIS
.TP 6
\fBctags\fP
[
.B \-aBdeFnNsStTuwWx
] [
.BI \-f " tagfile"
] [
.BI \-h " list"
]
.br
[
.BI \-i " types"
] [
.BI \-I " ignorelist"
] [
.BI \-L " listfile"
]
.br
[
.BI \-p " path"
] [
.BI \-o " tagfile"
] [
.B \--help
] [
.I file(s)
]

.br
.br
.TP 6
\fBetags\fP
[
.B \-adsStTx
] [
.BI \-f " tagfile"
] [
.BI \-h " list"
]
.br
[
.BI \-i " types"
] [
.BI \-I " ignorelist"
] [
.BI \-L " listfile"
]
.br
[
.BI \-p " path"
] [
.BI \-o " tagfile"
] [
.B \--help
] [
.I file(s)
]

.SH DESCRIPTION
The \fBctags\fP and \fBetags\fP programs (hereinafter collectively referred to
as \fBctags\fP, except where distinguished) generate an index (or "tag") file
of C language objects found in \fIfile(s)\fP that allows these items to be
quickly and easily located by a text editor or other utility. A "tag"
signifies a C language object for which an index entry is available (or,
alternatively, the index entry created for that object).
.PP
Alternatively, \fBctags\fP can generate a cross reference file which lists, in
human readable form, information about the various objects found in a set of C
language files.
.PP
Tag index files are supported by the \fBvi\fP(1) editor and its derivatives
(such as \fBvim\fP, \fBelvis\fP, \fBstevie\fP, and \fBxvi\fP) through the use
of the \fB:ta\fP command, and by the \fBemacs\fP editor through the use of the
\fBMeta-.\fP command, both of which locate the object associated with a name
appearing in a source file and jump to the file and line which defines the
name. The following types of tags are supported by \fBctags\fP:
.PP
.RS 4
macro definitions (i.e. names created by #define)
.br
enumerated values (i.e. the values inside enum{...})
.br
function definitions
.br
function prototypes or declarations (optional)
.br
class, enum, struct and union tags
.br
typedefs
.br
variables
.RE
.PP
.B Ctags
only generates tags for objects which have global scoping (file-wide
visibility). This means that, with the exception of macro definitions, only
objects defined outside of brace enclosed function blocks are candidates for a
tag. For every one of the qualified objects which are discovered in the source
files supplied to \fBctags\fP, a separate entry is added to the tag file.

When this program is invoked by the name \fBctags\fP, each line of the file is
in the the following format:
.PP
.PD 0
.RS 4
.TP 4
1.
tag name (a C language identifier)
.TP 4
2.
a single tab character
.TP 4
3.
the name of the file in which the object associated with the tag is located
.TP 4
4.
a single tab character
.TP 4
5.
an Ex command to locate the tag within the file; generally either a search
pattern (either /pattern/ or ?pattern?) or line number
.RE
.PD 1
.PP
Note that, unless changed via \fB-n\fP or \fB-N\fP, line numbers are only used
for tags from macro definitions, while patterns are used for all other tags.
.PP
When this program is invoked by the name \fBetags\fP, or with the \fB-e\fP
option, the output file is in a different format that is used by \fBemacs\fP(1).
.PP
Note that the name of each source file will be recorded in the tag file
exactly as it appears on the command line. Therefore, if the path you
specified on the command line was relative to some directory, then it will
be recorded in that same manner in the tag file.
.PP
This version of \fBctags\fP imposes no formatting requirements. Other versions
of ctags tended to rely upon certain formatting assumptions in order to help
it resolve coding dilemmas caused by preprocessor conditionals.
.PP
In general, \fBctags\fP tries to be smart about conditional preprocessor
directives. If a preprocessor conditional is encountered within a statement
which defines a tag, \fBctags\fP follows only the first branch of that
conditional (except in the special case of "#if 0", in which case it follows
only the last branch). The reason for this is that failing to pursue only one
branch can result in ambiguous syntax, as in the following example:
.PP
.RS
#ifdef TWO_ALTERNATIVES
.br
struct {
.br
#else
.br
union {
.br
#endif
.RS 4
short a;
.br
long b;
.RE
}
.RE
.PP
Both branches cannot be followed, or braces become unbalanced and \fBctags\fP
would be unable to make sense of the syntax.
.PP
If the application of this heuristic fails to properly parse a file,
generally due to complicated and inconsistent pairing within the conditionals,
\fBctags\fP will retry the file using a different heuristic which does not
selectively follow conditional preprocessor branches, but instead falls back
to relying upon a closing brace ("}") in column 1 as indicating the end of a
block once any brace imbalance is detected within a #if conditional.
.PP
\fBCtags\fP will also try to specially handle arguments lists enclosed in
double sets of parentheses in order to accept the following conditional
construct:
.PP
.RS
extern void foo __ARGS((int one, char two));
.RE
.PP
Any name immediately preceding the "((" will be automatically ignored and
the previous name will be used.
.PP
After creating or appending to the tag file, it is sorted by the tag name,
removing identical tag lines.
.PP
Note that the path recorded for filenames in the tag file and utilized by
the editor to search for tags are identical to the paths specified for
\fIfile(s)\fP on the command line. This means the if you want the paths for
files to be relative to some directory, you must invoke \fBctags\fP with the
same pathnames for \fIfile(s)\fP.

.SH OPTIONS
.PP
Despite the wealth of available options, defaults are set so that \fBctags\fP
is most commonly executed without any options (e.g. "ctags *.[ch]"), which
will create a tag file in the current directory for the specified files. The
options described below are provided merely to allow custom tailoring to meet
special needs.
.PP
Note that spaces separating options from their parameters are optional.
.TP 5
.B \--help
Prints to standard output a detailed usage description.
.TP 5
.B \-a
Append the tags to an existing tag file.
.TP 5
.B \-B
Use backward searching patterns (?...?).
.TP 5
.B \-d
Include macro definitions in the output file. This is equivalent to
.BI -i +d
and is supported only for backwards compatibility with other versions of
ctags. Use of \fB-i\fP is preferred.
.TP 5
.B \-e
Output tag file for use with Emacs. If this program is executed by the name
\fBetags\fP, this option is set by default. Selecting this option causes the
following options to be ignored: \fB-BFnNsuwW\fP.
.TP 5
.BI \-f " tagfile"
Use the name specified by \fBtagfile\fP for the tag file (default is "tags",
or "TAGS" when using the \fB-e\fP option). If \fItagfile\fP is specified as
"-", then the tag file is written to standard output instead.
.B Ctags
will stubbornly refuse to take orders if \fItagfile\fP exists and its first
line contains something other than a valid tags line. This will save your neck
if you mistakenly type "ctags -o *.c", which would otherwise overwrite your
first C file with the tags generated by the rest!
.TP 5
.B \-F
Use forward searching patterns (/.../) (default).
.TP 5
.BI \-h  " list"
Specifies a list of file extensions used for headers, separated by either
periods or commas. The default list is ".h.H.hpp.hxx.h++".
.TP 5
.BI \-i " types"
Specifies the list of tag types to include in the output file.
.I Types
is a group of letters designating the types of tags affected. Each letter or
group of letters may be preceded by either a '+' sign (default, if omitted) to
add it to those already included, a '-' sign to exclude it from the list (e.g.
to exclude a default tag type), or an '=' sign to include its corresponding
tag type at the exclusion of those not listed. The following tag types are
supported:
.RS 8
.PD 0
.TP 4
.I d
macro definitions
.TP 4
.I e
enumerated values (values inside enum{...})
.TP 4
.I f
function and method definitions
.TP 4
.I g
enum/struct/union tags (or new C++ types)
.TP 4
.I p
external function prototypes
.TP 4
.I t
typedefs
.TP 4
.I v
variable declarations
.RE
.RS 5
In addition, the following two modifiers are accepted:
.RE
.RS 8
.TP 4
.I P
Prefix static tags (if included) in the tag file with the filename in which
they appear, followed by a colon (Elvis style; not widely supported).
.TP 4
.I S
Include static tags (those not visible outside of a single source file).
Function and variable definitions are considered static only when their
definitions are preceded with the "static" keyword. All other types of tags
are considered static when they appear in a non-header file (see the -h
option).
.RE
.RS 5
The default value of \fItypes\fP is "=defgtvS" (i.e all tag types except for
function prototypes; include static tags but do not prefix them).
.RE
.PD 1
.TP 5
.BI \-I " ignorelist"
Reads a list of names which are to be ignored while generating tags for the
source files. The list may be supplied directly on the command line or found
in a separate file. Normally, the parameter
.I ignorelist
is a list of names to be ignored, each separated with a comma, a semicolon, or
white space (in which case the list should be quoted to keep the entire list
as one command line argument). The parameter
.I ignorelist
will be interpreted as a filename if its first character is given as either
a '.' or a pathname separator ('/' or '\\'). In order to specify a file found
in the current directory, use "./filename".
.RS 5
.PP
This feature is useful when preprocessor macros are used in such a way that
they cause syntactic confusion due to their presence. Some examples will
illustrate this point.
.RE
.PP
.RS
/* creates a global version string in module */
MODULE_VERSION("$Revision: 3.1 $")
.RE
.PP
.RS 5
In this example, the macro invocation looks to much like a function definition
because it is not followed by a semicolon (indeed, it could even be followed
by a global variable definition that would look exactly like a K&R style
function parameter declaration). In fact, this seeming function definition
would likely cause the rest of the file to be skipped over while trying to
complete the definition. Ignoring "MODULE_ID" would avoid such a problem.
.RE
.PP
.RS
int foo ARGDECL2(void *, ptr, long int, nbytes)
.RE
.PP
.RS 5
In this example, the macro "ARGDECL2" would be mistakenly interpreted to be
the name of the function instead of the correct name of "foo". Ignoring the
name "ARGDECL2" results in the correct behavior.
.RE
.TP 5
.BI \-L " listfile"
Read from
.I listfile
a list of file names for which tags should be generated. If
.I listfile
is specified as "-", then file names are read from standard input.
.TP 5
.B \-n
Places into the tag file line numbers in the source file where tags are located
rather than patterns to be searched for. This has three advantages:
.RS 5
.PD 0
.TP 4
1.
Significantly reduces the size of the resulting tag file.
.TP 4
2.
Eliminates failures to find tags because the line defining the tag has
changed, causing the pattern match to fail (note that some editors, such as
\fBvim\fP, are able to recover in many such instances).
.TP 4
3.
Eliminates finding identical matching, but incorrect, source lines (see
\fBBUGS\fP, below).
.PP
However, this option has one significant drawback: changes to the source files
can cause the line numbers recorded in the tag file to no longer correspond
to the lines in the source file, causing jumps to some tags to miss the target
definition by one or more lines. Basically, this option is best used when the
source code to which it is applied is not subject to change. See also the
\fB-N\fP option. Selecting this option causes the following options to be
ignored: \fB-BFN\fP.
.PD 1
.RE
.TP 5
.B \-N
Uses search patterns for all tags, rather than the line numbers usually used
for macro definitions. This has the advantage of not referencing obsolete
line numbers when lines have been added or removed since the tag file was
generated. See also the \fB-n\fP option. Selecting this option causes the
following options to be ignored: \fB-BF\fP.
.TP 5
.BI \-o " tagfile"
Alternative for \fB-f\fP.
.TP 5
.BI \-p " path"
Use \fIpath\fP as the default directory for all supplied source files (whether
supplied on the command line or in a file specified with the \fB-L\fP option),
unless the source file is already specified as an absolute path. Note that
this will not work together with wildcards in the filenames.
.TP 5
.B \-s
Include static tags in the output file, each prefixed with the name of
the file in which it appears followed by a colon. This is equivalent to
.BI -i +SP
and is supported only for backwards compatibility with older versions of
Elvis ctags. Use of \fB-i\fP is preferred.
.TP 5
.B \-S
Include static tags in the output file, but do not prefix them, thereby
making them appear the same as global tags. This is equivalent to
.BI -i +S-P
and is supported only for backwards compatibility with older versions of Elvis
ctags. Use of \fB-i\fP is preferred.
.TP 5
.B \-t
Include typedefs in the output file. This is equivalent to
.BI -i +t
and is supported only for backwards compatibility with other versions of
ctags. Use of
.B -i
is preferred.
.TP 5
.B \-T
Include typedefs and class/enum/struct/union tags in the output file.
This is equivalent to
.BI -i +tg
and is supported only for backwards compatibility with other versions of
ctags. Use of
.B -i
is preferred.
.TP 5
.B \-u
Unsorted; do not sort the tags. Please note that this disables the warning
messages normally enabled by
.BR \-W ,
because sorted tags are used to detect duplicate tags. Note also that
.BR vi (1)
requires sorted tags.
.TP 5
.B \-w
Exclude warnings about duplicate tags (default).
.TP 5
.B \-W
Generate warnings about duplicate tags.
.TP 5
.B \-x
Print a tabular, human-readable cross reference (xref) file to standard
output. The information contained in the output includes: the tag name; the
tag type; the line number, file name, and source line (with extra white space
condensed) of the file which defines the tag. No tag file is written and the
following options will be ignored: \fB-aBfFno\fP, and \fB-i\fP\fI+P\fP.
Example applications for this feature are generating a listing of all
functions (including statics) located in a source file (e.g. \fBctags
-xi\fP\fI=fS\fP \fIfile\fP), or generating a list of all externally visible
global variables located in a source file (e.g. \fBctags -xi\fP\fI=v file\fP).

.SH "ENVIRONMENT VARIABLES"
.TP 8
.B CTAGS
This variable, if found, will be assumed to contain a set of custom default
options which are read when \fBctags\fP starts, but before any command line
options are read. Options in this variable should be in the same form as those
on the command line. Command line options will override options specified in
this variable. Only options may be specified with this variable; no source
file names are read from its value.
.PP
.TP 8
.B ETAGS
Similar to the \fBCTAGS\fP variable above, this variable, if found, will be
read when \fBetags\fP starts. If this variable is not found, \fBetags\fP will
try to use \fBCTAGS\fP instead.

.SH "HOW TO USE WITH VI"
Vi will, by default, expect a tag file by the name "tags" in the current
directory. Once the tag file is built, the following commands exercise the tag
indexing feature:
.TP 12
.B vi -t tag
Start vi and position the cursor at the file and line where "tag" is defined.
.TP 12
.B Control-]
Find the tag under the cursor.
.TP 12
.B :ta tag
Find a tag.
.TP 12
.B Control-T
Return to previous location before jump to tag (not widely implemented).

.SH "HOW TO USE WITH GNU EMACS"
Emacs will, by default, expect a tag file by the name "TAGS" in the current
directory. Once the tag file is built, the following commands exercise the
tag indexing feature:
.TP 12
.B "Meta-x visit-tags-table"
Visit a TAGS file.
.TP 12
.B "Meta-."
Find a definition for a tag.  The default tag is the identifier under the
cursor.  There is name completion in the minibuffer; typing \fB"foo TAB"\fP
completes the identifier starting with `foo' (`foobar', for example) or lists
the alternatives.
.TP 12
.B "Meta-,"
Find the next definition for the tag.  Exact matches are found first, followed
by fuzzier matches.
.PP
For more commands, see the
.I Tags
topic in the Emacs info tree.

.SH BUGS
Support for C++ features is quite limited.
.PP
Because \fBctags\fP does not look inside brace enclosed function blocks, local
declarations of class/enum/struct/union tags and enumeration values within a
function will not have tags generated for them.
.PP
Note that because \fBctags\fP generates search patterns for non-macro tags, it
is entirely possible that the wrong line may be found by your editor if there
exists another, identical, line (whose context prevents it from generating a
tag) which is identical to the line containing the tag. The following example
demonstrates this condition:
.PP
.RS
int variable;

/* ... */
.br
void foo(variable)
.br
int variable;
.br
{
.RS 4
/* ... */
.RE
}
.RE
.PP
Depending upon which editor you use and where in the code you happen to be, it
is possible that the search pattern may locate the local parameter declaration
in foo() before it finds the actual global variable definition, since the
lines (and therefore their search patterns are identical). This can be avoided
by use of the \fB-n\fP option.
.PP
Because \fBctags\fP is neither a preprocessor nor a compiler, some complex
or obscure constructs can fool \fBctags\fP into either missing a tag or
improperly generating an inappropriate tag. In particular, the use of
preprocessor constructs which alter the textual syntax of C can fool
\fBctags\fP, as demonstrated by the following example:
.PP
.RS
#ifdef GLOBAL
.br
#define EXTERN
.br
#define INIT(assign)  assign
.br
#else
.br
#define EXTERN extern
.br
#define INIT(assign)
.br
#endif
.br

EXTERN BUF *firstbuf INIT(= NULL);
.RE
.PP
This looks too much like a declaration for a function called "INIT", which
returns a pointer to a typedef "firstbuf", rather than the actual variable
definition that it is, since this distinction can only be resolved by the
preprocessor. The moral of the story: don't do this if you want a tag
generated for it, or use the \fB-I\fP option to specify "INIT" as a keyword
to be ignored.

.SH FILES
.TP 10
.I tags
The default tag file created by \fBctags\fP.
.TP 10
.I TAGS
The default tag file created by \fBetags\fP.

.SH "SEE ALSO"
The official Exuberant Ctags web site at:
.PP
.RS 4
http://fly.hiwaay.net/~darren/ctags/
.RE
.PP
Also \fBex\fP(1), \fBvi\fP(1), \fBelvis\fP, or, better yet, \fBvim\fP, the
official editor of \fBctags\fP. For more information on \fBvim\fP, see the VIM
Pages web site at:
.PP
.RS 4
http://www.math.fu-berlin.de/~guckes/vim/
.RE

.SH AUTHOR
Darren Hiebert <darren@hiwaay.net>
.br
http://fly.hiwaay.net/~darren

.SH MOTIVATION
"Think ye at all times of rendering some service to every member of the human
race."
.PP
"All effort and exertion put forth by man from the fullness of his heart is
worship, if it is prompted by the highest motives and the will to do service
to humanity."
.PP
.RS 10
\-- From the Baha'i Writings

.SH CREDITS
This version of \fBctags\fP is derived from and inspired by the ctags program
by Steve Kirkendall (kirkenda@cs.pdx.edu) that comes with the Elvis vi clone
(though almost none of the original code remains).
.PP
Credit is also due Bram Moolenaar, the author of \fBvim\fP, who has devoted
so much of his time and energy both to developing the editor as a service to
others, and to helping the orphans of Uganda. 
.PP
The section entitled "HOW TO USE WITH GNU EMACS" was shamelessly stolen from
the man page for GNU \fBetags\fP.
